==> spread-5.7.0.72-py3-none-any.whl: spread_core/tests/equipment/shade/test_engineries.py <==
from typing import Any
from unittest import TestCase

from spread_core.mqtt.spread.address import ProviderAddress

from spread_core.bam.engineries import shade
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from spread_core.tests.equipment.utils import TestEquipmentAdapter, TestEngineryRecipe


class _TestAlarmSensors(TestCase):
    enginery_type = shade.Shade

    def setUp(self) -> None:
        self.adapter = TestEquipmentAdapter()
        self.recipe = TestEngineryRecipe(3)

        enginery = self.enginery_type(999)
        enginery.set_adapter(self.adapter)

        self.enginery = enginery

        self._set_recipes()

    def _set_recipes(self):
        self.enginery.set_recipe('SomeType', self.recipe)

    def _emulate_state(self, funit, value, provider_id):
        self.enginery.on_state(State(ProviderAddress(1, 'Manager', 2, 'Provider', provider_id, funit)), Variable(value))

    def _check_state(self, funit, value, provider_id=3, expect_state=True, expected_funit=None, expected_value: Any = '***useSetValue***'):
        self._emulate_state(funit, value, provider_id)

        if expected_funit is None:
            expected_funit = funit

        if expected_value == '***useSetValue***':
            expected_value = value

        self._check_value(expected_funit, expected_value)

        if expect_state:
            self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/{expected_funit}', expected_value)

        self._check_end_of_publishes()

    def _check_value(self, funit, expected_value):
        self.assertEqual(expected_value, self.enginery.values.get(funit))

    def _check_end_of_publishes(self):
        if self.adapter.queue:
            unexpected_publishes = f'Unexpected publishes ({len(self.adapter.queue)}):\n'
            unexpected_publishes += "\n".join(str(i) for i in self.adapter.queue)
            self.fail(unexpected_publishes)

    def _check_publish(self, _topic, _value, _retain=True, _delay=0):
        topic, variable, retain, delay = self.adapter.queue.popleft()
        self.assertEqual(_topic, topic)
        self.assertEqual(_value, variable.value)
        self.assertEqual(_retain, retain)
        self.assertEqual(_delay, delay)

    def test_recipe_appointment(self):
        recipe = TestEngineryRecipe(1001)
        self.enginery.set_recipe('SomeString', recipe)
        self.assertIs(self.enginery.recipe, recipe)

    def test_address(self):
        self.assertEqual(self.enginery.get_address('SomeFunit'), f'1/Equipment/{self.enginery_type.__name__}/999/SomeFunit')

    def get_address_broadcast(self):
        self.assertEqual(self.enginery.get_address_broadcast('SomeFunit'), f'1/Equipment/{self.enginery_type.__name__}/+/SomeFunit')

    def test_recipe_selecting(self):
        funit, recipe = self.enginery.recipe_for_funit('SomeFunit')
        self.assertEqual(funit, 'SomeFunit')
        self.assertEqual(recipe, self.recipe)

    def test_state_handler(self):
        self._check_state('PositionLevel', 5000)
        self._check_state('PositionLevel', 5000, expect_state=False)
        self._check_state('PositionLevel', None)

        self._check_state('Motion', 'Stopped')
        self._check_state('Motion', 'Opening')
        self._check_state('Motion', 'Closing')
        self._check_state('Motion', None)

    def test_invalidation(self):
        self.enginery.values['Funit1'] = 1
        self.enginery.values['Funit2'] = 'Funit2'
        self.enginery.values['Funit3'] = [1, 2, 3]

        self.enginery.set_invalid()

        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit1', None)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit2', None)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit3', None)

        self._check_end_of_publishes()

    def test_funit_PositionLevel(self):
        shade.PositionLevel(self.enginery, Variable(5000)).set()
        self._check_publish('Spread/Set/Provider/3/PositionLevel', 5000, False, 0)
        self._check_end_of_publishes()

        shade.PositionLevel(self.enginery, Variable(0)).set()
        self._check_publish('Spread/Set/Provider/3/PositionLevel', 0, False, 0)
        self._check_end_of_publishes()

    def test_funit_Motion(self):
        shade.Motion(self.enginery, Variable('Open')).set()
        self._check_publish('Spread/Set/Provider/3/Motion', 'Open', False, 0)

        shade.Motion(self.enginery, Variable('Close')).set()
        self._check_publish('Spread/Set/Provider/3/Motion', 'Close', False, 0)

        shade.Motion(self.enginery, Variable('Stop')).set()
        self._check_publish('Spread/Set/Provider/3/Motion', 'Stop', False, 0)

        funit = shade.Motion(self.enginery, Variable('SomeInvalidValue'))
        self.assertRaises(ValueError, funit.set)


class TestShutter(_TestAlarmSensors):
    enginery_type = shade.Shutter


class TestCurtain(_TestAlarmSensors):
    enginery_type = shade.Curtain


class TestBlind(_TestAlarmSensors):
    enginery_type = shade.Blind

    def _set_recipes(self):
        self.recipe2 = TestEngineryRecipe(4)
        self.enginery.set_recipe('Closeness', self.recipe)
        self.enginery.set_recipe('Angle', self.recipe2)

    def test_state_handler(self):
        self._check_state('PositionLevel', 0, provider_id=4, expected_funit='PositionAngle', expected_value=-9000)
        self._check_state('PositionLevel', 5000, provider_id=4, expected_funit='PositionAngle', expected_value=0)
        self._check_state('PositionLevel', 10000, provider_id=4, expected_funit='PositionAngle', expected_value=9000)
        self._check_state('PositionLevel', None, provider_id=4, expected_funit='PositionAngle')

        self._check_state('Motion', 'Opening', provider_id=4, expected_funit='Rotation', expected_value='CW')
        self._check_state('Motion', 'Closing', provider_id=4, expected_funit='Rotation', expected_value='CCW')
        self._check_state('Motion', 'SomeInvalidMotion', provider_id=4, expected_funit='Rotation', expected_value=None)
        self._check_state('Motion', 'Stopped', provider_id=4, expected_funit='Rotation', expected_value='Stopped')
        self._check_state('Motion', None, provider_id=4, expected_funit='Rotation')

    def test_recipe_appointment(self):
        recipe1 = TestEngineryRecipe(1001)
        recipe2 = TestEngineryRecipe(1002)
        self.assertRaises(ValueError, self.enginery.set_recipe, 'SomeString', recipe1)

        self.enginery.set_recipe('Closeness', recipe1)
        self.assertIs(self.enginery.recipe, recipe1)
        self.assertEqual(('Motion', recipe1), self.enginery.recipe_for_funit('Motion'))
        self.assertEqual(('PositionLevel', recipe1), self.enginery.recipe_for_funit('PositionLevel'))

        self.enginery.set_recipe('Angle', recipe2)
        self.assertEqual(('Motion', recipe2), self.enginery.recipe_for_funit('Rotation'))
        self.assertEqual(('PositionLevel', recipe2), self.enginery.recipe_for_funit('PositionAngle'))

    def test_funit_PositionAngle(self):
        shade.PositionAngle(self.enginery, Variable(-9000)).set()
        self._check_publish('Spread/Set/Provider/4/PositionLevel', 0, False, 0)
        self._check_end_of_publishes()

        shade.PositionAngle(self.enginery, Variable(9000)).set()
        self._check_publish('Spread/Set/Provider/4/PositionLevel', 10000, False, 0)
        self._check_end_of_publishes()

        funit = shade.PositionAngle(self.enginery, Variable(-9001))
        self.assertRaises(ValueError, funit.set)

        funit = shade.PositionAngle(self.enginery, Variable(9001))
        self.assertRaises(ValueError, funit.set)

    def test_funit_Rotation(self):
        shade.Rotation(self.enginery, Variable('CW')).set()
        self._check_publish('Spread/Set/Provider/4/Motion', 'Open', False, 0)
        self._check_value('Rotation', 'CW')
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Rotation', 'CW')

        shade.Rotation(self.enginery, Variable('CCW')).set()
        self._check_publish('Spread/Set/Provider/4/Motion', 'Close', False, 0)
        self._check_value('Rotation', 'CCW')
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Rotation', 'CCW')

        shade.Rotation(self.enginery, Variable('Stop')).set()
        self._check_publish('Spread/Set/Provider/4/Motion', 'Stop', False, 0)
        self._check_value('Rotation', 'Stopped')
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Rotation', 'Stopped')

        funit = shade.Rotation(self.enginery, Variable('Stop'))
        self.assertRaises(AssertionError, funit.set)

        funit = shade.Rotation(self.enginery, Variable('SomeInvalidValue'))
        self.assertRaises(ValueError, funit.set)

==> spread-5.8.1.80-py3-none-any.whl: spread_core/tests/equipment/shade/test_engineries.py <==
from typing import Any
from unittest import TestCase

from spread_core.mqtt.spread.address import ProviderAddress

from spread_core.bam.engineries import shade
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from spread_core.tests.equipment.utils import TestEquipmentAdapter, TestEngineryRecipe


class _TestShade(TestCase):
    enginery_type = shade.Shade

    def setUp(self) -> None:
        self.adapter = TestEquipmentAdapter()
        self.recipe = TestEngineryRecipe(3)

        enginery = self.enginery_type(999)
        enginery.set_adapter(self.adapter)

        self.enginery = enginery

        self._set_recipes()

    def _set_recipes(self):
        self.enginery.set_recipe('SomeType', self.recipe)

    def _emulate_state(self, funit, value, provider_id):
        self.enginery.on_state(State(ProviderAddress(1, 'Manager', 2, 'Provider', provider_id, funit)), Variable(value))

    def _check_state(self, funit, value, provider_id=3, expect_state=True, expected_funit=None, expected_value: Any = '***useSetValue***'):
        self._emulate_state(funit, value, provider_id)

        if expected_funit is None:
            expected_funit = funit

        if expected_value == '***useSetValue***':
            expected_value = value

        self._check_value(expected_funit, expected_value)

        if expect_state:
            self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/{expected_funit}', expected_value)

        self._check_end_of_publishes()

    def _check_value(self, funit, expected_value):
        self.assertEqual(expected_value, self.enginery.values.get(funit))

    def _check_end_of_publishes(self):
        if self.adapter.queue:
            unexpected_publishes = f'Unexpected publishes ({len(self.adapter.queue)}):\n'
            unexpected_publishes += "\n".join(str(i) for i in self.adapter.queue)
            self.fail(unexpected_publishes)

    def _check_publish(self, _topic, _value, _retain=True, _delay=0):
        topic, variable, retain, delay = self.adapter.queue.popleft()
        self.assertEqual(_topic, topic)
        self.assertEqual(_value, variable.value)
        self.assertEqual(_retain, retain)
        self.assertEqual(_delay, delay)

    def test_recipe_appointment(self):
        recipe = TestEngineryRecipe(1001)
        self.enginery.set_recipe('SomeString', recipe)
        self.assertIs(self.enginery.recipe, recipe)

    def test_address(self):
        self.assertEqual(self.enginery.get_address('SomeFunit'), f'1/Equipment/{self.enginery_type.__name__}/999/SomeFunit')

    def test_address_broadcast(self):
        self.assertEqual(self.enginery.get_address_broadcast(1, 'SomeFunit'), f'1/Equipment/{self.enginery_type.__name__}/+/SomeFunit')

    def test_recipe_selecting(self):
        funit, recipe = self.enginery.recipe_for_funit('SomeFunit')
        self.assertEqual(funit, 'SomeFunit')
        self.assertEqual(recipe, self.recipe)

    def test_state_handler(self):
        self._check_state('PositionLevel', 5000)
        self._check_state('PositionLevel', 5000, expect_state=False)
        self._check_state('PositionLevel', None)

        self._check_state('Motion', 'Stopped')
        self._check_state('Motion', 'Opening')
        self._check_state('Motion', 'Closing')
        self._check_state('Motion', None)

    def test_invalidation(self):
        self.enginery.values['Funit1'] = 1
        self.enginery.values['Funit2'] = 'Funit2'
        self.enginery.values['Funit3'] = [1, 2, 3]

        self.enginery.set_invalid()

        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit1', None)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit2', None)
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Funit3', None)

        self._check_end_of_publishes()

    def test_funit_PositionLevel(self):
        shade.PositionLevel(self.enginery, Variable(5000)).set()
        self._check_publish('Spread/Set/Provider/3/PositionLevel', 5000, False, 0)
        self._check_end_of_publishes()

        shade.PositionLevel(self.enginery, Variable(0)).set()
        self._check_publish('Spread/Set/Provider/3/PositionLevel', 0, False, 0)
        self._check_end_of_publishes()

    def test_funit_Motion(self):
        shade.Motion(self.enginery, Variable('Open')).set()
        self._check_publish('Spread/Set/Provider/3/Motion', 'Open', False, 0)

        shade.Motion(self.enginery, Variable('Close')).set()
        self._check_publish('Spread/Set/Provider/3/Motion', 'Close', False, 0)

        shade.Motion(self.enginery, Variable('Stop')).set()
        self._check_publish('Spread/Set/Provider/3/Motion', 'Stop', False, 0)

        funit = shade.Motion(self.enginery, Variable('SomeInvalidValue'))
        self.assertRaises(ValueError, funit.set)


class TestShutter(_TestShade):
    enginery_type = shade.Shutter


class TestCurtain(_TestShade):
    enginery_type = shade.Curtain


class TestBlind(_TestShade):
    enginery_type = shade.Blind

    def _set_recipes(self):
        self.recipe2 = TestEngineryRecipe(4)
        self.enginery.set_recipe('Closeness', self.recipe)
        self.enginery.set_recipe('Angle', self.recipe2)

    def test_state_handler(self):
        self._check_state('PositionLevel', 0, provider_id=4, expected_funit='PositionAngle', expected_value=-9000)
        self._check_state('PositionLevel', 5000, provider_id=4, expected_funit='PositionAngle', expected_value=0)
        self._check_state('PositionLevel', 10000, provider_id=4, expected_funit='PositionAngle', expected_value=9000)
        self._check_state('PositionLevel', None, provider_id=4, expected_funit='PositionAngle')

        self._check_state('Motion', 'Opening', provider_id=4, expected_funit='Rotation', expected_value='CW')
        self._check_state('Motion', 'Closing', provider_id=4, expected_funit='Rotation', expected_value='CCW')
        self._check_state('Motion', 'SomeInvalidMotion', provider_id=4, expected_funit='Rotation', expected_value=None)
        self._check_state('Motion', 'Stopped', provider_id=4, expected_funit='Rotation', expected_value='Stopped')
        self._check_state('Motion', None, provider_id=4, expected_funit='Rotation')

    def test_recipe_appointment(self):
        recipe1 = TestEngineryRecipe(1001)
        recipe2 = TestEngineryRecipe(1002)
        self.assertRaises(ValueError, self.enginery.set_recipe, 'SomeString', recipe1)

        self.enginery.set_recipe('Closeness', recipe1)
        self.assertIs(self.enginery.recipe, recipe1)
        self.assertEqual(('Motion', recipe1), self.enginery.recipe_for_funit('Motion'))
        self.assertEqual(('PositionLevel', recipe1), self.enginery.recipe_for_funit('PositionLevel'))

        self.enginery.set_recipe('Angle', recipe2)
        self.assertEqual(('Motion', recipe2), self.enginery.recipe_for_funit('Rotation'))
        self.assertEqual(('PositionLevel', recipe2), self.enginery.recipe_for_funit('PositionAngle'))

    def test_funit_PositionAngle(self):
        shade.PositionAngle(self.enginery, Variable(-9000)).set()
        self._check_publish('Spread/Set/Provider/4/PositionLevel', 0, False, 0)
        self._check_end_of_publishes()

        shade.PositionAngle(self.enginery, Variable(9000)).set()
        self._check_publish('Spread/Set/Provider/4/PositionLevel', 10000, False, 0)
        self._check_end_of_publishes()

        funit = shade.PositionAngle(self.enginery, Variable(-9001))
        self.assertRaises(ValueError, funit.set)

        funit = shade.PositionAngle(self.enginery, Variable(9001))
        self.assertRaises(ValueError, funit.set)

    def test_funit_Rotation(self):
        shade.Rotation(self.enginery, Variable('CW')).set()
        self._check_publish('Spread/Set/Provider/4/Motion', 'Open', False, 0)
        self._check_value('Rotation', 'CW')
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Rotation', 'CW')

        shade.Rotation(self.enginery, Variable('CCW')).set()
        self._check_publish('Spread/Set/Provider/4/Motion', 'Close', False, 0)
        self._check_value('Rotation', 'CCW')
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Rotation', 'CCW')

        shade.Rotation(self.enginery, Variable('Stop')).set()
        self._check_publish('Spread/Set/Provider/4/Motion', 'Stop', False, 0)
        self._check_value('Rotation', 'Stopped')
        self._check_publish(f'Spread/State/1/Equipment/{self.enginery_type.__name__}/999/Rotation', 'Stopped')

        funit = shade.Rotation(self.enginery, Variable('Stop'))
        self.assertRaises(AssertionError, funit.set)

        funit = shade.Rotation(self.enginery, Variable('SomeInvalidValue'))
        self.assertRaises(ValueError, funit.set)
